#ifndef _GSM1_H_
#define _GSM1_H_

//#ifdef HAVE_GSM_HEADER

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

#include "gsm.h"

#ifdef __cplusplus
}
#endif // __cplusplus

//#elif defined(HAVE_GSM_GSM_HEADER)
//#include <gsm/gsm.h>
//#endif

#include <ao/ao.h>


#define BUFFER_SAMPLES	8000
#define GSM_SAMPLES	160
#define	GSM_FRAME_LEN	33
#define	MSGSM_FRAME_LEN	65

#define LOG_WARNING 1

#define ast_log(type, fmt) printf(fmt)

struct gsm_translator_pvt {	/* both gsm2lin and lin2gsm */
	gsm gsma;
	int16_t buf[BUFFER_SAMPLES];	/* lin2gsm, temporary storage */
};


/*! \brief Data structure associated with a single frame of data
 */
struct ast_frame {
	/*! Kind of frame */
	//enum ast_frame_type frametype;
	int frametype;
	/*! Subclass, frame dependent */
	int subclass;				
	/*! Length of data */
	int datalen;				
	/*! Number of 8khz samples in this frame */
	int samples;				
	/*! Was the data malloc'd?  i.e. should we free it when we discard the frame? */
	int mallocd;				
	/*! The number of bytes allocated for a malloc'd frame header */
	size_t mallocd_hdr_len;
	/*! How many bytes exist _before_ "data" that can be used if needed */
	int offset;				
	/*! Optional source of frame for debugging */
	const char *src;				
	/*! Pointer to actual data */
	void *data;		
	/*! Global delivery time */		
	struct timeval delivery;
	/*! For placing in a linked list */
	//AST_LIST_ENTRY(ast_frame) frame_list;
	/*! Timing data flag */
	int has_timing_info;
	/*! Timestamp in milliseconds */
	long ts;
	/*! Length in milliseconds */
	long len;
	/*! Sequence number */
	int seqno;
};

struct ast_trans_pvt;

struct ast_trans_pvt {
	//struct ast_translator *t;
	struct ast_frame f;	/*!< used in frameout */
	int samples;		/*!< samples available in outbuf */
	int datalen;		/*!< actual space used in outbuf */
	void *pvt;		/*!< more private data, if any */
	char *outbuf;		/*!< the useful portion of the buffer */
	//plc_state_t *plc;	/*!< optional plc pointer */
	struct ast_trans_pvt *next;	/*!< next in translator chain */
	struct timeval nextin;
	struct timeval nextout;
};

int gsm_new(struct ast_trans_pvt *pvt);
void gsm_destroy_stuff(struct ast_trans_pvt *pvt);
int gsmtolin_framein(struct ast_trans_pvt *pvt, struct ast_frame *f);
int file_read(char *filename, char *data, int maxlen);
void mono_to_stereo(char *buffer, char *input, int input_size);

int audio_init(ao_device **device, char *init_card);
void audio_destroy(ao_device *device);
int audio_play(char *buffer, int buf_size, ao_device *device);

#endif // _GSM1_H_
